function optstruc = mkoptstruc() 
%MKOPTSTRUC   Creates a MATLAB 6-compatible optimset-structure 
% 
% Syntax 
%          options=mkoptstruc() 
% 
% Description 
%          This function provides a MATLAB 6 optimset work-alike. 
%          It generates an empty optimset-structure that can be 
%          passed to the high-level doptlin/optlinf functions, or 
%          to the lower-level lmmore function. 
%          Note that this function only generates a default optimset- 
%          structure. It is not capable to option-merging like the 
%          MATLAB 6 optimset function. It is recommended to use 
%          optimset if running MATLAB 6. 
% 
% Inputs 
%          None 
% 
% Outputs 
% options  The default optimset-structure 
% 
% Used by 
%          example 
% 
% See also 
%          optimset 
 
% Niek Bergboer, 2001
% Revised by Ivo Houtzager, 2007
% Copyright (c) 1996-2007, Delft Center of Systems and Control 
 
 optstruc = struct('ActiveConstrTol',[], ... 
       'DerivativeCheck',[], ... 
           'Diagnostics',[], ... 
         'DiffMaxChange',[], ... 
         'DiffMinChange',[], ... 
               'Display',[], ... 
     'GoalsExactAchieve',[], ... 
            'GradConstr',[], ... 
               'GradObj',[], ... 
               'Hessian',[], ... 
              'HessMult',[], ... 
           'HessPattern',[], ... 
            'HessUpdate',[], ... 
              'Jacobian',[], ... 
             'JacobMult',[], ... 
          'JacobPattern',[], ... 
            'LargeScale',[], ... 
    'LevenbergMarquardt',[], ... 
        'LineSearchType',[], ... 
           'MaxFunEvals',[], ... 
               'MaxIter',[], ... 
            'MaxPCGIter',[], ... 
            'MaxSQPIter',[], ... 
         'MeritFunction',[], ... 
             'MinAbsMax',[], ... 
        'Preconditioner',[], ... 
      'PrecondBandWidth',[], ... 
      'ShowStatusWindow',[], ... 
                'TolCon',[], ... 
                'TolFun',[], ... 
                'TolPCG',[], ... 
                  'TolX',[], ... 
              'TypicalX',[]); 
 
 
 

