clear all;
close all;
clc

%%
load('TurbineData_FxdBtm.mat');
Turb_data_FxdBtm = Turbine1_data;
clear Turbine1_data
load("DTU10MW_Triple_Data.mat");
load('SoftWind_Data.mat');
load('WindData_FxdBtm.mat');

valuestr = {'Azimuthal Position Blade 1 [deg]','Pitch Angle Blade 1 [deg]','Pitch Angle Blade 2 [deg]','Pitch Angle Blade 3 [deg]',...
    'Aerodynamic Thrust [N]','Aerodynamic Power [kW]','Thrust Coefficient [-]','Power Coefficient [-]',...
    'Rotational Speed [rpm]','Gen. HSS Torque [Nm]','Gen. Elec. Power [kW]','NP Trans. X_g [m]','NP Trans. Y_g [m]','NP Trans. Z_g [m]',...
    'NP Roll X_l [deg]','NP Pitch Y_l [deg]','NP Yaw Z_l [deg]','Time [s]','Y_c RootBend. Mom. (OOP) BLD 1 [Nm]',...
    'Y_c RootBend. Mom. (OOP) BLD 2 [Nm]','Y_c RootBend. Mom. (OOP) BLD 3 [Nm]',...
    'Total Tangential Load Blade 1 [N]','Total Tangential Load Blade 2 [N]','Total Tangential Load Blade 3 [N]',...
    'Total Normal Load Blade 1 [N]','Total Normal Load Blade 2 [N]','Total Normal Load Blade 3 [N]',...
    'Aerodynamic Force in Hub Y Direction [N]','Aerodynamic Force in Hub Z Direction [N]','Z_g Prescr. Platform Rot. [deg]',...
    'Platform Rot. Prescr. Z [m]','Lift Coefficient (at section) Blade 1 [-]','Lift Coefficient (at section) Blade 2 [-]'...
    ,'Lift Coefficient (at section) Blade 3 [-]','Total Velocity (at section) Blade 1 [m/s]','Total Velocity (at section) Blade 2 [m/s]'...
    ,'Total Velocity (at section) Blade 3 [m/s]'}  ;
%%

U_inf = 9;
D = 178.3;
Str = [0 0.1 0.15 0.2 0.25 0.3 0.35 0.4 0.45 0.5 0.6 0.7];
Freq = Str*U_inf/D;
Hel_Time = 1./Freq;

x = D*[-1 0 1 2 2.5 3 3.5 4 4.5 5 6];
y = 0 ;
z = 119 ;
[X,ST] = meshgrid(x(1:end),Str);

time = 8000*0.05:2:32000*0.05 ;
time_data = 10*0.05:0.5:32000*0.05 ;

TUcolors = [12, 35, 64 ; 
            0, 184, 200;
            0, 118, 194;
            111, 29, 119;
            239, 96, 163;
            165, 0, 52;
            224, 60, 49;
            237, 104, 66;
            255, 184, 28;
            108, 194, 74;
            0, 155, 119;
            0, 166, 214]/256;

Velocity_80_Triple = diff(squeeze(Turbine1_data_4deg(17,:,17))*pi/180)*(0.9*D/2) ;
Velocity_80_Triple = [Velocity_80_Triple Velocity_80_Triple(end)];

Velocity_80_Soft = diff(squeeze(Turbine_data(17,:,17))*pi/180)*(0.9*D/2) ;
Velocity_80_Soft = [Velocity_80_Soft Velocity_80_Soft(end)];

Velocities_4deg = permute(Velocities_4deg, [1 3 2]);
power_down_4deg = permute(power_down_4deg, [1 3 2]);
Vels_fxbtm = permute(Velocities, [1 3 2]);
power_down = permute(power_down, [1 3 2]);

means_vels_Triple = mean(Velocities_4deg(:,1:end,300:end),3);
means_power_Triple = mean(power_down_4deg(:,1:end,:),3)+ mean(squeeze(Turbine1_data_4deg(:,800:end,6)),2);

means_vels_Soft = mean(velocities(:,1:end,300:end),3);
means_power_Soft = mean(powers_down(:,1:end,:),3)+ mean(squeeze(Turbine_data(:,800:end,6)),2);

mean_vels_fxdbtm = mean(Vels_fxbtm(:,:,300:end),3);
mean_power_fxdbtm = mean(power_down(:,:,:),3)+mean(squeeze(Turb_data_FxdBtm(:,800:end,6)),2);

TripleVels = means_vels_Triple(19,:);

%% Figures
height_scale = 1.2;
width = 18.2;
height = width/height_scale;

fig = figure('Units', 'centimeters', 'pos', [-1 1 width height/height_scale],'color','w')
subplot('Position', [0.07, 0.52, 0.44, 0.42])
[M_wind2_fb,s_wind2_fb]= contourf(X(:,3:end)./D,ST(:,3:end),means_vels_Triple(1:12,3:end)./U_inf,0.3:0.05:0.9) ;
hold on
[wind2_fb_M,wind2_fb_s] = contour(X(:,3:end)./D,ST(:,3:end),means_vels_Triple(1:12,3:end)./U_inf,0.3:0.1:1,'k','LineWidth',1.25) ;
contour(X(:,3:end)./D,ST(:,3:end),means_vels_Triple(1:12,3:end)./U_inf,0.35:0.1:95,'k:','LineWidth',0.5) ;
text(1.2,0.65,2,'Baseline TripleSpar','Interpreter','latex','FontSize',10,'Color','white')
clabel(wind2_fb_M,wind2_fb_s,'Interpreter','latex','FontSize',10)
crameri nuuk;
clim([0.4 0.9])
xlim([1.01 6])
s_wind2_fb.LineColor = 'none' ;
grid on
ax = gca;
ax.FontSize = 10; 
set(gca,'TickLabelInterpreter','latex','XTickLabel',[])
ylabel('Strouhal [-]','Interpreter','latex','FontSize',12)

subplot('Position', [0.07, 0.085, 0.44, 0.42])
[M_pow2_fl,s_pow2_fl] = contourf(X(:,3:end)./D,ST(:,3:end),means_vels_Triple(13:24,3:end)./U_inf,0.3:0.05:0.9) ;
hold on
[wind2_fl_M,wind2_fl_s] = contour(X(:,3:end)./D,ST(:,3:end),means_vels_Triple(13:24,3:end)./U_inf,0.3:0.1:1,'k','LineWidth',1.25) ;
contour(X(:,3:end)./D,ST(:,3:end),means_vels_Triple(13:24,3:end)./U_inf,0.35:0.1:95,'k:','LineWidth',0.5) ;
text(1.2,0.65,2,'Optimized TripleSpar','Interpreter','latex','FontSize',10,'Color','white')
clabel(wind2_fl_M,wind2_fl_s,'Interpreter','latex','FontSize',10)
crameri nuuk;
clim([0.4 0.9])
xlim([1.01 6])
s_pow2_fl.LineColor = 'none' ;
grid on
ax = gca;
ax.FontSize = 10; 
set(gca,'TickLabelInterpreter','latex')
ylabel('Strouhal [-]','Interpreter','latex','FontSize',12)
xlabel('Distance in Rotor Diameters $$[D]$$','Interpreter','latex','FontSize',12)

subplot('Position', [0.07+0.45, 0.52, 0.44, 0.42])
[M_wind4_fb,s_wind4_fb]= contourf(X(:,3:end)./D,ST(:,3:end),means_vels_Soft(1:12,3:end)./U_inf,0.3:0.05:0.9) ;
hold on
[wind4_fb_M,wind4_fb_s] = contour(X(:,3:end)./D,ST(:,3:end),means_vels_Soft(1:12,3:end)./U_inf,0.3:0.1:1,'k','LineWidth',1.25) ;
contour(X(:,3:end)./D,ST(:,3:end),means_vels_Soft(1:12,3:end)./U_inf,0.35:0.1:95,'k:','LineWidth',0.5) ;
text(1.2,0.65,2,'Baseline Softwind','Interpreter','latex','FontSize',10,'Color','white')
clabel(wind4_fb_M,wind4_fb_s,'Interpreter','latex','FontSize',10)
crameri nuuk;
clim([0.4 0.9])
xlim([1.01 6])
s_wind4_fb.LineColor = 'none' ;
grid on
ax = gca;
ax.FontSize = 10; 
set(gca,'TickLabelInterpreter','latex','XTickLabel',[],'YTickLabel',[])

subplot('Position', [0.07+0.45, 0.085, 0.44, 0.42])
[M_wind4_fl,s_wind4_fl] = contourf(X(:,3:end)./D,ST(:,3:end),means_vels_Soft(13:24,3:end)./U_inf,0.3:0.05:0.9) ;
hold on
[wind4_fl_M,wind4_fl_s] = contour(X(:,3:end)./D,ST(:,3:end),means_vels_Soft(13:24,3:end)./U_inf,0.3:0.1:1,'k','LineWidth',1.25) ;
contour(X(:,3:end)./D,ST(:,3:end),means_vels_Soft(13:24,3:end)./U_inf,0.35:0.1:95,'k:','LineWidth',0.5) ;
text(1.2,0.65,2,'Optimized Softwind','Interpreter','latex','FontSize',10,'Color','white')
clabel(wind4_fl_M,wind4_fl_s,'Interpreter','latex','FontSize',10)
crameri nuuk;
clim([0.4 0.9])
xlim([1.01 6])
s_wind4_fl.LineColor = 'none' ;
grid on
ax = gca;
ax.FontSize = 10; 
set(gca,'TickLabelInterpreter','latex','YTickLabel',[])
xlabel('Distance in Rotor Diameters $$[D]$$','Interpreter','latex','FontSize',12)
sgtitle('Normalized Wind Speed','Interpreter','latex','FontSize',12)
exportgraphics(gcf,'Torque2024_Figures/WindSpeed_Triple_Soft.pdf','ContentType','vector')

%%
fig = figure('Units', 'centimeters', 'pos', [-1 1 width height/2],'color','w')
subplot('Position', [0.07, 0.145, 0.45, 0.812])
[M_wind2_fb,s_wind2_fb]= contourf(X(:,3:end)./D,ST(:,3:end),means_power_Triple(13:24,3:end)./means_power_Triple(1:12,3:end),0.9:0.025:1.1) ;
hold on
[wind2_fb_M,wind2_fb_s] = contour(X(:,3:end)./D,ST(:,3:end),means_power_Triple(13:24,3:end)./means_power_Triple(1:12,3:end),0.9:0.05:1.1,'k','LineWidth',1.25) ;
contour(X(:,3:end)./D,ST(:,3:end),means_power_Triple(13:24,3:end)./means_power_Triple(1:12,3:end),0.825:0.05:1.125,'k:','LineWidth',0.5) ;
text(1.2,0.715,2,'Relative Power TripleSpar Farm','Interpreter','latex','FontSize',10,'Color','black')
clabel(wind2_fb_M,wind2_fb_s,'Interpreter','latex','FontSize',10)
crameri nuuk;
clim([0.9 1.1])
xlim([1.0 6])
s_wind2_fb.LineColor = 'none' ;
grid on
ax = gca;
ax.FontSize = 10; 
set(gca,'TickLabelInterpreter','latex')
ylabel('Strouhal [-]','Interpreter','latex','FontSize',12)
xlabel('Distance in Rotor Diameters $$[D]$$','Interpreter','latex','FontSize',12)

subplot('Position', [0.07+0.46, 0.145, 0.45, 0.812])
[M_wind2_fb,s_wind2_fb]= contourf(X(:,3:end)./D,ST(:,3:end),means_power_Soft(13:24,3:end)./means_power_Soft(1:12,3:end),0.9:0.025:1.1) ;
hold on
[wind2_fb_M,wind2_fb_s] = contour(X(:,3:end)./D,ST(:,3:end),means_power_Soft(13:24,3:end)./means_power_Soft(1:12,3:end),0.9:0.05:1.1,'k','LineWidth',1.25) ;
contour(X(:,3:end)./D,ST(:,3:end),means_power_Soft(13:24,3:end)./means_power_Soft(1:12,3:end),0.825:0.05:1.125,'k:','LineWidth',0.5) ;
text(1.2,0.715,2,'Relative Power Softwind Farm','Interpreter','latex','FontSize',10,'Color','black')
clabel(wind2_fb_M,wind2_fb_s,'Interpreter','latex','FontSize',10)
crameri nuuk;
clim([0.9 1.1])
xlim([1.01 6])
s_wind2_fb.LineColor = 'none' ;
grid on
ax = gca;
ax.FontSize = 10; 
set(gca,'TickLabelInterpreter','latex','YTickLabel',[])
xlabel('Distance in Rotor Diameters $$[D]$$','Interpreter','latex','FontSize',12)
exportgraphics(gcf,'Torque2024_Figures/Relative_Power_Triple_Soft.pdf','ContentType','vector')